/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright 2013 Red Hat, Inc.
 */

#include "config.h"

#include <locale.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <glib/gi18n-lib.h>

#include <nm-client.h>
#include <nm-connection.h>
#include <nm-remote-settings.h>
#include <nm-utils.h>

#include "nmt-newt.h"

#include "nmtui.h"
#include "nmt-connection-utils.h"
#include "nm-editor-bindings.h"

NMClient *nm_client;
NMRemoteSettings *nm_settings;

static void
connections_read (NMRemoteSettings *settings,
                  gpointer          user_data)
{
	gboolean *got_connections = user_data;

	*got_connections = TRUE;
}

gboolean sleep_on_startup = FALSE;
gboolean noinit = FALSE;

GOptionEntry entries[] = {
	{ "sleep", 's', 0, G_OPTION_ARG_NONE, &sleep_on_startup,
	  "Sleep on startup", NULL },
	{ "noinit", 'n', 0, G_OPTION_ARG_NONE, &noinit,
	  "Don't initialize newt", NULL },
	{ NULL }
};

int
main (int argc, char **argv)
{
	NMConnection *conn = NULL;
	gboolean got_connections = FALSE;
	GOptionContext *opts;
	GError *error = NULL;

	setlocale (LC_ALL, "");

	opts = g_option_context_new (NULL);
	g_option_context_add_main_entries (opts, entries, NULL);

	if (!g_option_context_parse (opts, &argc, &argv, &error)) {
		g_printerr ("Could not parse arguments: %s\n",
		            error->message);
		exit (1);
	}
	g_option_context_free (opts);

	if (argc > 2) {
		g_printerr ("Usage: %s [connection]\n", argv[0]);
		exit (1);
	}

	nm_client = nm_client_new ();

	nm_settings = nm_remote_settings_new (NULL);
	g_signal_connect (nm_settings, NM_REMOTE_SETTINGS_CONNECTIONS_READ,
	                  G_CALLBACK (connections_read), &got_connections);
	while (!got_connections)
		g_main_context_iteration (NULL, TRUE);

	if (argc == 2) {
		if (nm_utils_is_uuid (argv[1]))
			conn = NM_CONNECTION (nm_remote_settings_get_connection_by_uuid (nm_settings, argv[1]));
		else {
			GSList *conns, *iter;

			conns = nm_remote_settings_list_connections (nm_settings);
			for (iter = conns; iter; iter = iter->next) {
				NMConnection *candidate = iter->data;

				if (!strcmp (argv[1], nm_connection_get_id (candidate))) {
					conn = candidate;
					break;
				}
			}
			g_slist_free (conns);
		}

		if (!conn) {
			g_printerr ("%s: no such connection '%s'\n", argv[0], argv[1]);
			exit (1);
		}
	}

	if (sleep_on_startup)
		sleep (5);

	if (!noinit)
		nmt_newt_init ();
	nm_editor_bindings_init ();

	if (conn)
		nmt_edit_connection (conn);
	else
		nmt_main_connection_list ();

	nmt_newt_finished ();

	g_object_unref (nm_client);
	g_object_unref (nm_settings);

	return 0;
}
